import numpy as np
import matplotlib.pyplot as plt
from astropy.table import Table
import math
import matplotlib.ticker as ticker
from scipy import interpolate
from scipy.optimize import curve_fit

cluster = input('cluster name : ')
fn = 'total_{0}.dat'.format(cluster)
dat = Table.read(fn, format='ascii')
mem_cut_prob = 0.70

if cluster == 'cz30':
    x_center = 2794.3582
    y_center = 1276.5077
elif cluster == 'be34':
    x_center = 2954.6035
    y_center = 1120.938
elif cluster == 'be75':
    x_center = 2752.0689
    y_center = 1304.837
elif cluster == 'be76':
    x_center = 3075.0307
    y_center = 1292.5035

dat['dist'] = np.sqrt((dat['X'] - x_center)**2 + (dat['Y'] - y_center)**2)

if cluster == 'be75':
    xUpperLimit = 4063 - x_center
    xLowerLimit = x_center
    yUpperLimit = 4063 - y_center
    yLowerLimit = y_center
else:
    xUpperLimit = 4064 - x_center
    xLowerLimit = x_center
    yUpperLimit = 4064 - y_center
    yLowerLimit = y_center

length = np.array([xUpperLimit, xLowerLimit, yUpperLimit, yLowerLimit])
step_size = 0.5 * 60 / 0.289

bootstrap_count = 0 
bootstrap_radius = []
while bootstrap_count < 20000:
    boot_idx = np.random.choice(np.arange(len(dat)), size = len(np.squeeze(np.where(dat['probs_final'] >= mem_cut_prob))))
    new_dat = dat[boot_idx]
    radius_range = np.arange(0.5, max(length)*(0.289/60)+0.1, 0.5)
    count_area = np.zeros(len(radius_range))
    area = np.zeros(len(radius_range))
    star = np.zeros(len(radius_range))
    mem = np.zeros(len(radius_range))
    for i in  np.arange(len(radius_range)):
        if i == 0:
            area[i] = np.pi*(radius_range[i]**2)
        else:
            area[i] = np.pi*(radius_range[i]**2 - radius_range[i-1]**2)
    
    if xUpperLimit > yUpperLimit:
        circ_cut = math.trunc(yUpperLimit / step_size)
        count_area[:circ_cut] += 1/4
    else:
        circ_cut = math.trunc(xUpperLimit / step_size)
        count_area[:circ_cut] += 1/4
    if xLowerLimit > yUpperLimit:
        circ_cut = math.trunc(yUpperLimit / step_size)
        count_area[:circ_cut] += 1/4
    else:
        circ_cut = math.trunc(xLowerLimit / step_size)
        count_area[:circ_cut] += 1/4
    if xLowerLimit > yLowerLimit:
        circ_cut = math.trunc(yLowerLimit / step_size)
        count_area[:circ_cut] += 1/4
    else:
        circ_cut = math.trunc(xLowerLimit / step_size)
        count_area[:circ_cut] += 1/4
    if xUpperLimit > yLowerLimit:
     circ_cut = math.trunc(yLowerLimit / step_size)
     count_area[:circ_cut] += 1/4
    else:
        circ_cut = math.trunc(xUpperLimit / step_size)
        count_area[:circ_cut] += 1/4
    
    area = area * count_area

    for i in np.arange(len(new_dat)):
        if new_dat['X'][i] >= x_center and new_dat['Y'][i] >= y_center:
            if xUpperLimit > yUpperLimit:
                if new_dat['dist'][i] / step_size <= math.trunc(yUpperLimit / step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1
            else:
                if new_dat['dist'][i]/step_size <= math.trunc(xUpperLimit / step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1
        elif new_dat['X'][i] < x_center and new_dat['Y'][i] >= y_center:
            if xLowerLimit > yUpperLimit:
                if new_dat['dist'][i]/step_size <= math.trunc(yUpperLimit/step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1
            else:
                if new_dat['dist'][i]/step_size <= math.trunc(xLowerLimit/step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1
        elif new_dat['X'][i] < x_center and new_dat['Y'][i] < y_center:
            if xLowerLimit > yLowerLimit:
                if new_dat['dist'][i]/step_size <= math.trunc(yLowerLimit/step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1
            else:
                if new_dat['dist'][i]/step_size <= math.trunc(xLowerLimit/step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1
        elif new_dat['X'][i] >= x_center and new_dat['Y'][i] < y_center:
            if xUpperLimit > yLowerLimit:
                if new_dat['dist'][i]/step_size <= math.trunc(yLowerLimit/step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1
            else:
                if new_dat['dist'][i]/step_size <= math.trunc(xUpperLimit/step_size):
                    star[math.trunc(new_dat['dist'][i] / step_size)] += 1
                    if new_dat['probs_final'][i] >= mem_cut_prob:
                        mem[math.trunc(new_dat['dist'][i] / step_size)] += 1

    num_per_arcmin = star / area
    mem_per_arcmin = mem / area
    """
    test = [num_per_arcmin[i]+num_per_arcmin[i-1] for i in np.arange(len(num_per_arcmin)) if i != 0]
    test2 = [mem_per_arcmin[i]+mem_per_arcmin[i-1] for i in np.arange(len(mem_per_arcmin)) if i != 0]
    test = np.array(test)
    test2 = np.array(test2)
    
    f_tot = interpolate.interp1d(radius_range[1:], test, fill_value='extrapolate')
    f_mem = interpolate.interp1d(radius_range[1:], test2, fill_value='extrapolate')

    xnew = np.arange(1, np.max(radius_range)+0.1, 0.1)
    """
    test = [num_per_arcmin[i]+num_per_arcmin[i-1] for i in np.arange(len(num_per_arcmin)) if i != 0]
    test2 = [mem_per_arcmin[i]+mem_per_arcmin[i-1] for i in np.arange(len(mem_per_arcmin)) if i != 0]
    test.insert(0,num_per_arcmin[0])
    test2.insert(0,mem_per_arcmin[0])
    test = np.array(test)
    test2 = np.array(test2)
    
    f_tot = interpolate.interp1d(radius_range, test, fill_value='extrapolate')
    f_mem = interpolate.interp1d(radius_range, test2, fill_value='extrapolate')
    
    xnew = np.arange(0.5, np.max(radius_range)+0.1, 0.1)
    
    tot_new = f_tot(xnew)
    mem_new = f_mem(xnew)

    frac = mem_new / tot_new

    radius = np.nanargmin(np.abs(frac - 0.5))

    """
    f_tot = interpolate.interp1d(radius_range, num_per_arcmin)
    f_mem = interpolate.interp1d(radius_range, mem_per_arcmin)
    
    xnew = np.arange(0.5, np.max(radius_range)+0.1, 0.1)
    tot_new = f_tot(xnew)
    mem_new = f_mem(xnew)
    
    #frac = mem_per_arcmin / num_per_arcmin
    frac = mem_new / tot_new
    radius = np.nanargmin(np.abs(frac - 0.5))
   """ 
    #bootstrap_radius.append(radius_range[radius])
    bootstrap_radius.append(xnew[radius])
    bootstrap_count += 1

def gaussian(x, mean, amplitude, standard_deviation):
    return amplitude * np.exp( - (x - mean)**2 / (2*standard_deviation ** 2))

fig = plt.figure()
ax = fig.add_subplot(111)
x_bin_heights, x_bin_borders, _ = ax.hist(bootstrap_radius, bins=40)
x_bin_centers = x_bin_borders[:-1] + np.diff(x_bin_borders) / 2
x_popt, _ = curve_fit(gaussian, x_bin_centers, x_bin_heights, p0=[1., 0., 1.])
x_interval_for_fit = np.linspace(x_bin_borders[0], x_bin_borders[-1], 10000)
ax.plot(x_interval_for_fit, gaussian(x_interval_for_fit, *x_popt), c='r')
ax.axvline(x=x_popt[0]-x_popt[2], ls='--', c='grey')
ax.axvline(x=x_popt[0]+x_popt[2], ls='--', c='grey')
plt.savefig(f'rad_err_{cluster}.pdf')
print(x_popt[0], x_popt[2])

#ax.set_xlim(0.5, 3)
